@@ -1,15 +1,16 @@
 from django.apps import apps
 from django.contrib.auth import get_user_model, login, logout
 from django.utils.translation import gettext as _
-from rest_framework import viewsets, permissions, exceptions, routers
+from rest_framework import viewsets, permissions, exceptions, routers, status
 from rest_framework.decorators import action
+from rest_framework.permissions import AllowAny
 from rest_framework.response import Response
-from xently.core.loading import get_classes, get_class
+from xently.core.loading import get_classes
 
+from xauth.accounts.permissions import IsSuperuser, IsOwner
 from xauth.authentication import PasswordResetRequestAuthentication
 from xauth.internal_settings import AUTH_APP_LABEL
 
-IsOwner = get_class(f"{AUTH_APP_LABEL}.permissions", "IsOwner")
 (
     SecurityQuestionSerializer,
     ProfileSerializer,
@@ -37,7 +38,7 @@ __all__ = ["AccountViewSet", "SecurityQuestionViewSet"]
 class SecurityQuestionViewSet(viewsets.ModelViewSet):
     serializer_class = SecurityQuestionSerializer
     queryset = apps.get_model(AUTH_APP_LABEL, "SecurityQuestion").objects.all()
-    permission_classes = [permissions.IsAdminUser]  # TODO: consider superuser only
+    permission_classes = [IsSuperuser]
 
 
 class AccountViewSet(viewsets.ModelViewSet):
@@ -51,14 +52,25 @@ class AccountViewSet(viewsets.ModelViewSet):
         kwargs.setdefault("basename", routers.SimpleRouter().get_default_basename(self.__class__))
         super().__init__(*args, **kwargs)
 
+    def perform_create(self, serializer):
+        obj = serializer.save()
+        if self.action == "signup" and not obj.is_verified:
+            self.do_request_verification_code(obj)
+
     def get_queryset(self):
+        if self.action == "request_verification_code":
+            # When requesting for a verification code the authentication credentials (token) may expire. And given
+            # the essence of the request is to renew the token while getting a new code in the process, we need to
+            # allow for a user object to be retrieved from the request URL without requiring any form of
+            # authentication prior.
+            return super().get_queryset()
         return get_user_model().objects.filter(pk=self.request.user.pk)
 
     def get_object(self):
         lookup_key = self.lookup_url_kwarg or self.lookup_field
         if lookup_key not in self.kwargs or self.action == "request_temporary_password":
             # When requesting a temporary password, a persisted (i.e. references the same) user instance is
-            # required to signal the generation of the password reset token. Therefore our best bet is on
+            # required to signal the generation of the password reset token. Therefore, our best bet is on
             # the user object from the request
             obj = self.request.user
             self.check_object_permissions(self.request, obj)
@@ -84,7 +96,12 @@ class AccountViewSet(viewsets.ModelViewSet):
         self.serializer_class = self.__class__.serializer_class
         return super().retrieve(request, *args, **kwargs)
 
-    @action(methods=["POST"], detail=False, permission_classes=[permissions.AllowAny])
+    @action(
+        detail=False,
+        methods=["POST"],
+        authentication_classes=[],
+        permission_classes=[permissions.AllowAny],
+    )
     def signup(self, request, *args, **kwargs):
         return self.create(request, *args, **kwargs)
 
@@ -100,11 +117,19 @@ class AccountViewSet(viewsets.ModelViewSet):
 
     def do_request_verification_code(self, user):
         """This can be overridden to by projects to for example send SMS and/or email"""
-        user.request_verification(send_mail=True, request=self.request)
+        user.request_verification(send_email=True, request=self.request)
 
-    @action(detail=True, url_path="request-verification-code", serializer_class=None)
+    @action(
+        detail=True,
+        authentication_classes=[],
+        permission_classes=[AllowAny],
+        url_path="request-verification-code",
+    )
     def request_verification_code(self, request, *args, **kwargs):
-        self.do_request_verification_code(request.user)
+        user = self.get_object()
+        if user.is_verified:
+            return Response(status=status.HTTP_208_ALREADY_REPORTED)
+        self.do_request_verification_code(user)
         return self.retrieve(request, *args, **kwargs)
 
     @action(methods=["POST"], detail=True, url_path="verify-account", serializer_class=AccountVerificationSerializer)
@@ -116,19 +141,16 @@ class AccountViewSet(viewsets.ModelViewSet):
 
     def do_request_temporary_password(self, user):
         """This can be overridden to by projects to for example send SMS and/or email"""
-        user.request_password_reset(send_mail=True, request=self.request)
+        user.request_password_reset(send_email=True, request=self.request)
 
     @action(
-        detail=True,
+        detail=False,
         methods=["POST"],
         url_path="request-temporary-password",
         serializer_class=PasswordResetRequestSerializer,
         authentication_classes=[PasswordResetRequestAuthentication],
     )
     def request_temporary_password(self, request, *args, **kwargs):
-        lookup_kwarg_value = self.kwargs[self.lookup_url_kwarg or self.lookup_field]
-        if type(lookup_kwarg_value)(getattr(request.user, self.lookup_field)) != lookup_kwarg_value:
-            raise exceptions.PermissionDenied
         self.do_request_temporary_password(request.user)
         response = self.retrieve(request, *args, **kwargs)
         request.user.unflag_password_reset()
@@ -140,7 +162,8 @@ class AccountViewSet(viewsets.ModelViewSet):
         if serializer.is_valid(raise_exception=True) and self.request.user.reset_password(**serializer.validated_data):
             return self.retrieve(request, *args, **kwargs)
         is_change = serializer.validated_data["is_change"]
-        raise exceptions.ValidationError(_(f"Invalid {'old' if is_change else 'temporary'} password."))
+        error_message = _(f"Invalid {'old' if is_change else 'temporary'} password.")
+        raise exceptions.ValidationError({"old_password": [error_message]})
 
     @action(
         detail=True,
@@ -161,6 +184,8 @@ class AccountViewSet(viewsets.ModelViewSet):
         try:
             if serializer.is_valid(raise_exception=True) and request.user.activate_account(**serializer.validated_data):
                 return self.retrieve(request, *args, **kwargs)
-        except AttributeError:
-            raise AttributeError("Make sure you applied `models.UserActivationMixin` to your `settings.AUTH_USER_MODEL`")
+        except AttributeError as e:
+            raise AttributeError(
+                "Make sure you applied `models.UserActivationMixin` to your `settings.AUTH_USER_MODEL`"
+            ) from e
         return exceptions.ValidationError(_("Wrong security question answer"))
