@@ -1,13 +1,13 @@
-__all__ = ["REST_FRAMEWORK", "AUTH_USER_MODEL", "XENTLY_DYNAMIC_CLASS_LOADER"]
-
 REST_FRAMEWORK = {
     "DEFAULT_AUTHENTICATION_CLASSES": [
-        "xauth.authentication.JWTTokenAuthentication",
+        "xauth.authentication.JWTAuthentication",
         "rest_framework.authentication.BasicAuthentication",
         "rest_framework.authentication.SessionAuthentication",
     ],
 }
 
-AUTH_USER_MODEL = "xauth.User"
+AUTH_USER_MODEL = "accounts.User"
 
 XENTLY_DYNAMIC_CLASS_LOADER = "xauth.loaders.class_loader"
+
+XENTLY_DYNAMIC_CLASS_LOADER_MODULE_PREFIX = "xauth"
